import 'dart:async';

import 'package:lori_sdk/data/database/interfaces/cartDao.dart';
import 'package:lori_sdk/data/models/cart/cart.dart';
import 'package:lori_sdk/data/models/singleton/cartSingleton.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';

class MobileCartDao implements CartDao {
  Database db;

  Future open() async {
    if (db == null || !db.isOpen) {
      var databasesPath = await getDatabasesPath();
      String path = join(databasesPath, 'loriCart.db');
      db = await openDatabase(path, version: 7,
          onCreate: (Database db, int version) async {
        await db.execute('''
        create table $tableName (
          $columnId text primary key,
          $columnCpf text,
          $columnDiscountValue double,
          $columnFreightValue double,
          $columnFeeValue double,
          $columnSubTotal double,
          $columnTotal double,
          $columnSubTotalSeparated double,
          $columnTotalSeparated double,
          $columnMaxInstallments integer,
          $columnInstallments integer,
          $columnFreight text,
          $columnDeliveryAddress text,
          $columnIsDelivery integer,
          $columnDeliveryTime integer,
          $columnLatitude double,
          $columnLongitude double,
          $columnCep text,
          $columnCpfInvoice text,
          $columnCpfPickup text,
          $columnModality text,
          $columnItems text,
          $columnSeller text,
          $columnPaymentMethod text,
          $columnSupportedPaymentMethods text,
          $columnVoucher text,
          $columnIsCvvValidated integer,
          $columnHasFeeInstallment integer,
          $columnGiftsQuantityAllowed integer,
          $columnAvailableGiftItems text,
          $columnChangesFromLastUpdate text,
          $columnMessages text,
          $columnIsPickupOnly integer
        )
      ''');
      }, onUpgrade: (Database db, int oldVersion, int newVersion) async {
        if (newVersion == 2) {
          await db.execute('''
            alter table $tableName ADD $columnHasFeeInstallment integer;
          ''');
          await db.execute('''
            alter table $tableName ADD $columnFeeValue double;
          ''');
        }
        if (newVersion == 3) {
          await db.execute('''
          alter table $tableName ADD $columnSupportedPaymentMethods text;
          ''');
        }
        if (newVersion == 4) {
          await db.execute('''
          alter table $tableName ADD $columnGiftsQuantityAllowed integer default '0';
          ''');
          await db.execute('''
          alter table $tableName ADD $columnAvailableGiftItems text;
          ''');
          await db.execute('''
          alter table $tableName ADD $columnChangesFromLastUpdate text;
          ''');
        }
        if (newVersion == 5) {
          await db.execute('''
          alter table $tableName ADD $columnModality text;
          ''');
        }
        if(newVersion == 6){
          await db.execute('''
          alter table $tableName ADD $columnMessages text;
          ''');
        }
        if(newVersion == 7){
          await db.execute('''
          alter table $tableName ADD $columnIsPickupOnly integer default '0';
          ''');
        }

      });
    }
  }

  Future<Cart> get() async {
    await open();
    List<Map> maps = await db.query(tableName);
    if (maps != null && maps.length > 0) {
      var cart = Cart();
      cart.fromMap(maps.first);
      return cart;
    }
    return null;
  }

  Future<void> insertOrUpdate(Cart cart) async {
    var storedResponse = await get();
    if (storedResponse == null) {
      return insert(cart);
    } else {
      return update(cart);
    }
  }

  Future<void> insert(Cart cart) async {
    await open();
    await db.insert(tableName, cart.toMap());
    CartSingleton().cartChanged(cart);
    return cart;
  }

  Future<void> update(Cart cart) async {
    await open();
    CartSingleton().cartChanged(cart);
    return db.update(tableName, cart.toMap(),
        where: '$columnId = ?', whereArgs: [cart.id]);
  }

  Future<void> deleteAll() async {
    await open();
    return db.delete(
      tableName,
    );
  }
}

CartDao getCartDao() => MobileCartDao();
