import 'dart:async';
import 'dart:convert';

import 'package:lori_sdk/data/database/interfaces/productDao.dart';
import 'package:lori_sdk/data/models/product/product.dart';
import 'package:lori_sdk/data/models/singleton/productSingleton.dart';
import 'package:shared_preferences/shared_preferences.dart';

const int kVersion1 = 1;

class WebProductDao implements ProductDao {
  static const _SAW_PRODUCTS = 'x_products';

  SharedPreferences db;

  Future open() async {
    if (db == null) {
      db = await SharedPreferences.getInstance();
    }
  }

  Future<void> insertOrUpdate(Product product) async {
    var storedProducts = await get();
    Product readProduct;
    if (storedProducts != null) {
      readProduct =
          storedProducts.firstWhere((sp) => sp.id == product.id, orElse: () {
        return null;
      });
    }
    if (readProduct == null) {
      return await insert(product);
    } else {
      return await update(product);
    }
  }

  Future<void> insert(Product product) async {
    product.lastTimestamp = DateTime.now().millisecondsSinceEpoch;
    await open();
    List<Product> list = await _all();
    list.add(product);
    await db.setString(
        _SAW_PRODUCTS, jsonEncode(list.map((item) => item.toJson()).toList()));
    return ProductSingleton().productsChanged(await get());
  }

  Future<void> update(Product product) async {
    product.lastTimestamp = DateTime.now().millisecondsSinceEpoch;
    await open();
    List<Product> list = await _all();
    for (int i = 0; i < list.length; i++) {
      if (list[i].id == product.id) {
        list[i] = product;
        break;
      }
    }
    await db.setString(
        _SAW_PRODUCTS, jsonEncode(list.map((item) => item.toJson()).toList()));
    return ProductSingleton().productsChanged(await get());
  }

  Future<List<Product>> get() async {
    await open();
    List<Product> products = await _all();
    return products.length == 0 ? null : products;
  }

  Future<void> deleteAll() async {
    await open();
    await db.setString(_SAW_PRODUCTS, null);
    return;
  }

  Future<List<Product>> _all() async {
    await open();
    String sawProductsInJson = db.getString(_SAW_PRODUCTS);
    List<Product> responses = [];
    if (sawProductsInJson != null) {
      List<Map> responsesMap = List.from(jsonDecode(sawProductsInJson));
      for (var map in responsesMap) {
        responses.add(Product.fromJson(map));
      }
    }
    return responses;
  }

  @override
  Future<void> remove(Product product) {
    throw UnimplementedError();
  }
}

ProductDao getProductDao() => WebProductDao();
