import 'package:intl/intl.dart';
import 'package:json_annotation/json_annotation.dart';

part 'userPromotion.g.dart';

@JsonSerializable()
class UserPromotion {
  UserPromotion();

  /// The id of this Promotion on the external server
  String externalId;

  /// The name of this Promotion
  String name;

  /// A descriptions of this Promotion
  String description;

  /// Not used by now
  String barcode;

  /// Some additional information about this promo
  String additionalInfo;

  /// The URL of the Image to be displayed at the promotion
  String imageUrl;

  int remainingMaxPerCustomer;


  /// The type of the promotion can be either "personalized" or "massive"
  @JsonKey(name: "promotionType")
  UserPromotionType type = UserPromotionType.personalized;

  /// The mechanic of the promotion interferes with the page layout
  ///   and which of the values fields are gonna be used.
  UserPromotionMechanic mechanic = UserPromotionMechanic.other;

  /// Whether this promotion is activated or not
  bool activated;

  /// Whether this promotion needs to be activated before it can be used
  bool needActivation;

  /// Since when the promo is valid
  String startDate;

  /// Until when the promo is valid
  String endDate;
  
  /// How many days until the promo expires
  int get daysToExpire {
    try {
      final formattedDate = DateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'", 'pt-BR')
          .parseStrict(this.endDate);
      Duration dur = formattedDate.difference(DateTime.now());
      return dur.inDays;
    }
    catch (e) {
      return 0;
    }

  }

  /// VALUES
  /// Those values are based on the mechanic, so some of them can be null
  ///   in a request and not null on others.

  /// The fixed amount of discount of this promo
  double discountAmount;

  /// The percentage of discount of this promo
  double discountPercent;

  /// The final price for the product, after the promo is activated
  double finalPrice;

  /// The regular price of the product, before the promo is activated
  double originalPrice;

  /// The minimal amount to be spent on the purchase to make the discount valid
  double minAmount;

  factory UserPromotion.fromJson(Map<String, dynamic> json) =>
      _$UserPromotionFromJson(json);

  Map<String, dynamic> toJson() => _$UserPromotionToJson(this);
}

enum UserPromotionMechanic {
  @JsonValue("promote_regular_price")
  promote_regular_price,

  @JsonValue("reduced_price")
  reduced_price,

  @JsonValue("money_off")
  money_off,

  @JsonValue("percent_off")
  percent_off,

  @JsonValue("spend_and_get")
  spend_and_get,

  @JsonValue("other")
  other
}

enum UserPromotionType {
  @JsonValue("personalized")
  personalized,

  @JsonValue("massive")
  massive
}